@@ -44,6 +44,8 @@
 #include <gedit-debug.h>
 #include <gedit2.h>
 #include <gedit-document.h>
+#include <gedit-utils.h>
+#include <gedit-menus.h>
 
 enum
 {
@@ -85,8 +87,25 @@ window_destroyed (GtkObject *obj,  void **window_pointer)
 
 	if (window_pointer != NULL)
 	{
+		GList *top_windows;
+        	gedit_debug (DEBUG_PLUGINS, "");
+
+        	top_windows = gedit_get_top_windows ();
+
+        	while (top_windows)
+        	{
+			BonoboUIComponent *ui_component;
+			
+			ui_component = gedit_get_ui_component_from_window (
+					BONOBO_WINDOW (top_windows->data));
+			
+			gedit_menus_set_verb_state (ui_component, "/commands/" MENU_ITEM_NAME, 0);
+
+			top_windows = g_list_next (top_windows);
+		}
+
 		g_free (*window_pointer);
-		*window_pointer = NULL;
+		*window_pointer = NULL;	
 	}	
 }
 
@@ -132,7 +151,7 @@ void taglist_window_show ()
 
 	gtk_tooltips_set_tip (gtk_tooltips_new (),
 			GTK_COMBO (tag_list_window->tag_groups_combo)->entry,
-			_("Select the group of tab you want to use."),
+			_("Select the group of tags you want to use"),
 			NULL);
 	
 	gtk_editable_set_editable (GTK_EDITABLE (
@@ -152,11 +171,21 @@ void taglist_window_show ()
 
 	/* Create tree view */
 	tag_list_window->tags_list = gtk_tree_view_new ();
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tag_list_window->tags_list), TRUE);
+
+	gedit_utils_set_atk_name_description (tag_list_window->tag_groups_combo, 
+							_("Tag Groups Combo"), NULL);
+	gedit_utils_set_atk_name_description (tag_list_window->tags_list, 
+							_("Tags Name List"), NULL);
+	gedit_utils_set_atk_relation (tag_list_window->tag_groups_combo, tag_list_window->tags_list, 
+							ATK_RELATION_CONTROLLER_FOR);
+	gedit_utils_set_atk_relation (tag_list_window->tags_list, tag_list_window->tag_groups_combo, 
+							ATK_RELATION_CONTROLLED_BY);
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tag_list_window->tags_list), FALSE);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tag_list_window->tags_list), FALSE);
 
 	gtk_tooltips_set_tip (gtk_tooltips_new (),
 			tag_list_window->tags_list,
-			_("Double-click on a tag to insert it in the current document."),
+			_("Double-click on a tag to insert it in the current document"),
 			NULL);
 
 	g_signal_connect_after (G_OBJECT (tag_list_window->tags_list), "row_activated",
@@ -197,6 +226,15 @@ void taglist_window_show ()
 
 }
 
+void 
+taglist_window_close ()
+{
+	gedit_debug (DEBUG_PLUGINS, "");
+
+	if (tag_list_window != NULL)
+		gtk_widget_destroy (GTK_WIDGET (tag_list_window->window));
+}
+
 static void
 populate_tag_groups_combo (void)
 {
@@ -389,24 +427,6 @@ tag_list_key_press_event_cb (GtkTreeView *tag_list, GdkEventKey *event)
 		return FALSE;
 	}
 
-	if (event->keyval == GDK_F1)
-	{
-		GError *error = NULL;
-
-		gedit_debug (DEBUG_PLUGINS, "F1 Pressed");
-
-		gnome_help_display ("gedit.xml", "gedit-use-plugins", &error);
-	
-		if (error != NULL)
-		{
-			g_warning (error->message);
-	
-			g_error_free (error);
-		}
-
-		return FALSE;
-	}
-
 	return FALSE;
 }
 
@@ -453,10 +473,28 @@ insert_tag (Tag *tag, gboolean focus_to_document)
 static gboolean 
 tag_list_window_key_press_event_cb (GtkTreeView *tag_list, GdkEventKey *event)
 {
-	if ((event->keyval == GDK_F4) && (event->state & GDK_MOD1_MASK))
+	if ((event->keyval == 'w') && (event->state & GDK_CONTROL_MASK))
 	{
-		gtk_widget_destroy (GTK_WIDGET (tag_list_window->window));
+		taglist_window_close ();
 		return TRUE;
+	}
+
+	if (event->keyval == GDK_F1)
+	{
+		GError *error = NULL;
+
+		gedit_debug (DEBUG_PLUGINS, "F1 Pressed");
+
+		gnome_help_display ("gedit.xml", "gedit-use-plugins", &error);
+	
+		if (error != NULL)
+		{
+			g_warning (error->message);
+	
+			g_error_free (error);
+		}
+
+		return FALSE;
 	}	
 
 	return FALSE;
