@@ -62,9 +62,9 @@ BonoboUIVerb gedit_verbs [] = {
 	BONOBO_UI_VERB ("SearchReplace", gedit_cmd_search_replace),
 	BONOBO_UI_VERB ("SearchGoToLine", gedit_cmd_search_goto_line),
 	BONOBO_UI_VERB ("SettingsPreferences", gedit_cmd_settings_preferences),
+	BONOBO_UI_VERB ("DocumentsMoveToNewWindow", gedit_cmd_documents_move_to_new_window),
 	BONOBO_UI_VERB ("HelpContents", gedit_cmd_help_contents),
 	BONOBO_UI_VERB ("About", gedit_cmd_help_about),
-	BONOBO_UI_VERB ("PluginsManager", gedit_cmd_tools_plugin_manager),
 
 	BONOBO_UI_VERB_END
 };
@@ -102,7 +102,7 @@ gchar* gedit_menus_no_docs_sensible_verbs [] = {
 	"/commands/SearchFindAgain",
 	"/commands/SearchReplace",
 	"/commands/SearchGoToLine",
-
+	"/commands/DocumentsMoveToNewWindow",
 	NULL
 };
 
@@ -113,7 +113,9 @@ gchar* gedit_menus_untitled_doc_sensible_verbs [] = {
 };
 
 gchar* gedit_menus_not_modified_doc_sensible_verbs [] = {
+	/*
 	"/commands/FileSave",
+	*/
 	"/commands/FileRevert",
 
 	NULL
@@ -338,4 +340,88 @@ gedit_menus_add_menu_item_toggle_all (const gchar *path,
 	}
 }
 
+void
+gedit_menus_add_submenu (BonoboWindow *window, const gchar *path,
+		     const gchar *name, const gchar *label)
+{
+	BonoboUIComponent *ui_component;
+	gchar *item_path;
+
+	g_return_if_fail (window != NULL);
+	g_return_if_fail (path != NULL);
+	g_return_if_fail (label != NULL);
+	
+	item_path = g_strconcat (path, name, NULL);
+	ui_component = gedit_get_ui_component_from_window (BONOBO_WINDOW (window));
+	if (!bonobo_ui_component_path_exists (ui_component, item_path, NULL)) {
+		gchar *xml;
+
+		xml = g_strdup_printf ("<submenu name=\"%s\""
+				       " _label=\"%s\" />", 
+				       name, label);
+
+		bonobo_ui_component_set_translate (ui_component, path,
+						   xml, NULL);
+
+		g_free (xml);
+	}
+
+	g_free (item_path);
+}
+
+void
+gedit_menus_add_submenu_all (const gchar *path, const gchar *name, const gchar *label)
+{
+	GList* top_windows;
+	
+	top_windows = gedit_get_top_windows ();
+	g_return_if_fail (top_windows != NULL);
+       
+	while (top_windows)
+	{
+		BonoboWindow* window = BONOBO_WINDOW (top_windows->data);
+
+		gedit_menus_add_submenu (window, path, name, label);
+		
+		top_windows = g_list_next (top_windows);
+	}
+
+}
+
+void
+gedit_menus_remove_submenu (BonoboWindow *window, const gchar *path,  const gchar *name)
+{
+	BonoboUIComponent *ui_component;
+	gchar *item_path;
+
+	g_return_if_fail (window != NULL);
+	g_return_if_fail (path != NULL);
+	g_return_if_fail (name != NULL);
+
+	item_path = g_strconcat (path, name, NULL);
+	ui_component = gedit_get_ui_component_from_window (BONOBO_WINDOW (window));
+
+	if (bonobo_ui_component_path_exists (ui_component, item_path, NULL)) 
+		bonobo_ui_component_rm (ui_component, item_path, NULL);
+
+	g_free (item_path);
+}
+
+void
+gedit_menus_remove_submenu_all (const gchar *path, const gchar *name)
+{
+	GList* top_windows;
+	
+	top_windows = gedit_get_top_windows ();
+	g_return_if_fail (top_windows != NULL);
+       
+	while (top_windows)
+	{
+		BonoboWindow* window = BONOBO_WINDOW (top_windows->data);
+
+		gedit_menus_remove_submenu (window, path, name);
+		
+		top_windows = g_list_next (top_windows);
+	}
+}
 
