@@ -28,13 +28,17 @@
  * See the ChangeLog files for a list of changes. 
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "gedit-undo-manager.h"
 #include "gedit-debug.h"
-#include "gedit-prefs.h"
+#include "gedit-prefs-manager.h"
 #include "gedit-marshal.h"
 
 typedef struct _GeditUndoAction  		GeditUndoAction;
@@ -43,7 +47,7 @@ typedef struct _GeditUndoDeleteAction		GeditUndoDeleteAction;
 
 typedef enum {
 	GEDIT_UNDO_ACTION_INSERT,
-	GEDIT_UNDO_ACTION_DELETE,
+	GEDIT_UNDO_ACTION_DELETE
 } GeditUndoActionType;
 
 /* 
@@ -93,6 +97,8 @@ struct _GeditUndoManagerPrivate
 	gboolean	can_redo;
 
 	gint		running_not_undoable_actions;
+
+	gint		num_of_groups;
 };
 
 enum {
@@ -122,7 +128,6 @@ static void gedit_undo_manager_check_list_size 		(GeditUndoManager *um);
 
 static gboolean gedit_undo_manager_merge_action 	(GeditUndoManager *um, 
 		                                         GeditUndoAction *undo_action);
-static gint gedit_undo_manager_get_number_of_groups 	(GeditUndoManager *um);
 
 static GObjectClass 	*parent_class 				= NULL;
 static guint 		undo_manager_signals [LAST_SIGNAL] 	= { 0 };
@@ -136,13 +141,13 @@ gedit_undo_manager_get_type (void)
     	{
       		static const GTypeInfo our_info =
       		{
-        		sizeof (GeditDocumentClass),
+        		sizeof (GeditUndoManagerClass),
         		NULL,		/* base_init */
         		NULL,		/* base_finalize */
         		(GClassInitFunc) gedit_undo_manager_class_init,
         		NULL,           /* class_finalize */
         		NULL,           /* class_data */
-        		sizeof (GeditDocument),
+        		sizeof (GeditUndoManager),
         		0,              /* n_preallocs */
         		(GInstanceInitFunc) gedit_undo_manager_init
       		};
@@ -206,6 +211,8 @@ gedit_undo_manager_init (GeditUndoManager *um)
 	um->priv->can_redo = FALSE;
 
 	um->priv->running_not_undoable_actions = 0;
+
+	um->priv->num_of_groups = 0;
 }
 
 static void
@@ -279,7 +286,8 @@ gedit_undo_manager_new (GeditDocument* document)
 	return um;
 }
 
-void gedit_undo_manager_begin_not_undoable_action (GeditUndoManager *um)
+void 
+gedit_undo_manager_begin_not_undoable_action (GeditUndoManager *um)
 {
 	gedit_debug (DEBUG_UNDO, "");
 
@@ -289,19 +297,50 @@ void gedit_undo_manager_begin_not_undoable_action (GeditUndoManager *um)
 	++um->priv->running_not_undoable_actions;
 }
 
-void gedit_undo_manager_end_not_undoable_action (GeditUndoManager *um)
+static void 
+gedit_undo_manager_end_not_undoable_action_internal (GeditUndoManager *um)
 {
 	gedit_debug (DEBUG_UNDO, "");
 
 	g_return_if_fail (GEDIT_IS_UNDO_MANAGER (um));
 	g_return_if_fail (um->priv != NULL);
 
-	if (um->priv->running_not_undoable_actions <= 0)
-		um->priv->running_not_undoable_actions = 0;
-	else
-		--um->priv->running_not_undoable_actions;
+	g_return_if_fail (um->priv->running_not_undoable_actions > 0);
+	
+	--um->priv->running_not_undoable_actions;
 }
 
+void 
+gedit_undo_manager_end_not_undoable_action (GeditUndoManager *um)
+{
+	gedit_debug (DEBUG_UNDO, "");
+
+	g_return_if_fail (GEDIT_IS_UNDO_MANAGER (um));
+	g_return_if_fail (um->priv != NULL);
+
+	gedit_undo_manager_end_not_undoable_action_internal (um);
+
+	if (um->priv->running_not_undoable_actions == 0)
+	{	
+		gedit_undo_manager_free_action_list (um);
+	
+		um->priv->next_redo = -1;	
+
+		if (um->priv->can_undo)
+		{
+			um->priv->can_undo = FALSE;
+			g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_UNDO], 0, FALSE);
+		}
+
+		if (um->priv->can_redo)
+		{
+			um->priv->can_redo = FALSE;
+			g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_REDO], 0, FALSE);
+		}
+	}
+}
+
+
 gboolean
 gedit_undo_manager_can_undo (const GeditUndoManager *um)
 {
@@ -347,18 +386,27 @@ gedit_undo_manager_undo (GeditUndoManager *um)
 		switch (undo_action->action_type)
 		{
 			case GEDIT_UNDO_ACTION_DELETE:
+				gedit_document_set_cursor (
+					um->priv->document, 
+					undo_action->action.delete.start);
+
 				gedit_document_insert_text (
 					um->priv->document, 
 					undo_action->action.delete.start, 
 					undo_action->action.delete.text,
 					strlen (undo_action->action.delete.text));
-				break;
+
+								break;
 			case GEDIT_UNDO_ACTION_INSERT:
 				gedit_document_delete_text (
 					um->priv->document, 
 					undo_action->action.insert.pos, 
 					undo_action->action.insert.pos + 
 						undo_action->action.insert.chars); 
+
+				gedit_document_set_cursor (
+					um->priv->document, 
+					undo_action->action.insert.pos);
 				break;
 
 			default:
@@ -368,7 +416,7 @@ gedit_undo_manager_undo (GeditUndoManager *um)
 
 	} while (undo_action->order_in_group > 1);
 
-	gedit_undo_manager_end_not_undoable_action (um);
+	gedit_undo_manager_end_not_undoable_action_internal (um);
 	
 	if (!um->priv->can_redo)
 	{
@@ -408,9 +456,18 @@ gedit_undo_manager_redo (GeditUndoManager *um)
 					um->priv->document, 
 					undo_action->action.delete.start, 
 					undo_action->action.delete.end); 
+
+				gedit_document_set_cursor (
+					um->priv->document,
+					undo_action->action.delete.start);
+
 				break;
 				
 			case GEDIT_UNDO_ACTION_INSERT:
+				gedit_document_set_cursor (
+					um->priv->document,
+					undo_action->action.insert.pos);
+
 				gedit_document_insert_text (
 					um->priv->document, 
 					undo_action->action.insert.pos, 
@@ -433,7 +490,7 @@ gedit_undo_manager_redo (GeditUndoManager *um)
 		
 	} while ((undo_action != NULL) && (undo_action->order_in_group > 1));
 
-	gedit_undo_manager_end_not_undoable_action (um);
+	gedit_undo_manager_end_not_undoable_action_internal (um);
 
 	if (um->priv->next_redo < 0)
 	{
@@ -470,7 +527,7 @@ gedit_undo_manager_free_action_list (GeditUndoManager *um)
 		GeditUndoAction *undo_action = 
 			(GeditUndoAction *)(g_list_nth_data (um->priv->actions, n));
 
-		gedit_debug (DEBUG_UNDO, "Free action (tye %s) %d/%d", 
+		gedit_debug (DEBUG_UNDO, "Free action (type %s) %d/%d", 
 				(undo_action->action_type == GEDIT_UNDO_ACTION_INSERT) ? "insert":
 				"delete", n, len);
 
@@ -481,6 +538,9 @@ gedit_undo_manager_free_action_list (GeditUndoManager *um)
 		else
 			g_return_if_fail (FALSE);
 
+		if (undo_action->order_in_group == 1)
+			--um->priv->num_of_groups;
+
 		g_free (undo_action);
 	}
 
@@ -553,6 +613,8 @@ gedit_undo_manager_delete_range_handler (GtkTextBuffer *buffer, GtkTextIter *sta
 	gedit_undo_manager_add_action (um, undo_action);
 
 	g_free (undo_action.action.delete.text);
+
+	gedit_debug (DEBUG_UNDO, "DONE");
 }
 
 static void 
@@ -612,6 +674,9 @@ gedit_undo_manager_add_action (GeditUndoManager *um, GeditUndoAction undo_action
 		
 		++um->priv->actions_in_current_group;
 		action->order_in_group = um->priv->actions_in_current_group;
+
+		if (action->order_in_group == 1)
+			++um->priv->num_of_groups;
 	
 		um->priv->actions = g_list_prepend (um->priv->actions, action);
 	}
@@ -630,6 +695,7 @@ gedit_undo_manager_add_action (GeditUndoManager *um, GeditUndoAction undo_action
 		g_signal_emit (G_OBJECT (um), undo_manager_signals [CAN_REDO], 0, FALSE);
 	}
 
+	gedit_debug (DEBUG_UNDO, "DONE");
 }
 
 static void 
@@ -657,6 +723,9 @@ gedit_undo_manager_free_first_n_actions (GeditUndoManager *um, gint n)
 		else
 			g_return_if_fail (FALSE);
 
+		if (undo_action->order_in_group == 1)
+			--um->priv->num_of_groups;
+
 		g_free (undo_action);
 
 		um->priv->actions = g_list_delete_link (um->priv->actions, um->priv->actions);
@@ -669,15 +738,19 @@ gedit_undo_manager_free_first_n_actions (GeditUndoManager *um, gint n)
 static void 
 gedit_undo_manager_check_list_size (GeditUndoManager *um)
 {
+	gint undo_levels;
+	
 	gedit_debug (DEBUG_UNDO, "TODO");
 
 	g_return_if_fail (GEDIT_IS_UNDO_MANAGER (um));
 	g_return_if_fail (um->priv != NULL);
 	
-	if (gedit_settings->undo_levels < 1)
+	undo_levels = gedit_prefs_manager_get_undo_actions_limit ();
+	
+	if (undo_levels < 1)
 		return;
 
-	if (gedit_undo_manager_get_number_of_groups (um) > gedit_settings->undo_levels)
+	if (um->priv->num_of_groups > undo_levels)
 	{
 		GeditUndoAction *undo_action;
 		GList* last;
@@ -694,6 +767,9 @@ gedit_undo_manager_check_list_size (GeditUndoManager *um)
 			else
 				g_return_if_fail (FALSE);
 
+			if (undo_action->order_in_group == 1)
+				--um->priv->num_of_groups;
+
 			g_free (undo_action);
 
 			um->priv->actions = g_list_delete_link (um->priv->actions, last);
@@ -703,8 +779,7 @@ gedit_undo_manager_check_list_size (GeditUndoManager *um)
 			undo_action = (GeditUndoAction*) last->data;
 
 		} while ((undo_action->order_in_group > 1) || 
-			 (gedit_undo_manager_get_number_of_groups (um) > 
-			  gedit_settings->undo_levels));
+			 (um->priv->num_of_groups > undo_levels));
 	}	
 }
 
@@ -834,21 +909,4 @@ gedit_undo_manager_merge_action (GeditUndoManager *um, GeditUndoAction *undo_act
 	return TRUE;
 }
 
-static gint
-gedit_undo_manager_get_number_of_groups (GeditUndoManager *um)
-{
-	GList *list = um->priv->actions;
-  	gint num = 0;
-
-	gedit_debug (DEBUG_UNDO, "");
-  
-  	while (list)
-    	{
-      		if (((GeditUndoAction* ) list->data)->order_in_group == 1)
-			++num;
-      		list = list->next;
-    	}
-  
-  	return num;
-}
 
