@@ -58,6 +58,9 @@ struct _GeditView
 struct _GeditViewClass
 {
 	GtkVBoxClass parent_class;
+
+	void (* populate_popup) 	(GeditView    *view,
+					 GtkMenu      *menu);
 };
 
 
@@ -69,20 +72,29 @@ void 		gedit_view_cut_clipboard 	(GeditView *view);
 void 		gedit_view_copy_clipboard 	(GeditView *view);
 void 		gedit_view_paste_clipboard	(GeditView *view);
 void 		gedit_view_delete_selection	(GeditView *view);
-void 		gedit_view_select_all 		(GeditView *view);
 
 void		gedit_view_scroll_to_cursor 	(GeditView *view);
 
 GeditDocument*	gedit_view_get_document		(const GeditView *view);
 
 void 		gedit_view_set_colors 		(GeditView* view, 
-						 GdkColor* backgroud, GdkColor* text);
+						 gboolean def,
+						 GdkColor* backgroud, GdkColor* text,
+						 GdkColor* selection, GdkColor* sel_text);
 
 void 		gedit_view_set_font		(GeditView* view,
+						 gboolean def,
 						 const gchar* font_name);
 
 void		gedit_view_set_wrap_mode 	(GeditView* view, GtkWrapMode wrap_mode);
 void		gedit_view_set_tab_size  	(GeditView* view, gint tab_size);
 
+void		gedit_view_show_line_numbers 	(GeditView* view, gboolean visible);
+
+void		gedit_view_set_cursor_position_statusbar (GeditView *view, GtkWidget* status);
+void		gedit_view_set_overwrite_mode_statusbar  (GeditView *view, GtkWidget* status);
+
+GtkTextView    *gedit_view_get_gtk_text_view 	(const GeditView *view);
+
 #endif /* __GEDIT_VIEW_H__ */
 
